package javax.swing.text;

import java.awt.Shape;
import javax.swing.event.DocumentEvent;
import java.awt.Container;
import java.awt.Toolkit;
import java.awt.FontMetrics;
import java.awt.Color;
import java.awt.Font;

public class LabelView extends GlyphView implements TabableView
{
    private Font font;
    private Color fg;
    private Color bg;
    private boolean underline;
    private boolean strike;
    private boolean superscript;
    private boolean subscript;
    
    public LabelView(Element element) {
        super(element);
    }
    
    final void sync() {
        if (this.font == null) {
            this.setPropertiesFromAttributes();
        }
    }
    
    protected void setUnderline(boolean underline) {
        this.underline = underline;
    }
    
    protected void setStrikeThrough(boolean strike) {
        this.strike = strike;
    }
    
    protected void setSuperscript(boolean superscript) {
        this.superscript = superscript;
    }
    
    protected void setSubscript(boolean subscript) {
        this.subscript = subscript;
    }
    
    protected void setBackground(Color bg) {
        this.bg = bg;
    }
    
    protected void setPropertiesFromAttributes() {
        AttributeSet attributes = this.getAttributes();
        if (attributes != null) {
            Document document = this.getDocument();
            if ((document instanceof StyledDocument)) {
                StyledDocument styledDocument = (StyledDocument)document;
                this.font = styledDocument.getFont(attributes);
                this.fg = styledDocument.getForeground(attributes);
                if (attributes.isDefined(StyleConstants.Background)) {
                    this.bg = styledDocument.getBackground(attributes);
                }
                else {
                    this.bg = null;
                }
                this.setUnderline(StyleConstants.isUnderline(attributes));
                this.setStrikeThrough(StyleConstants.isStrikeThrough(attributes));
                this.setSuperscript(StyleConstants.isSuperscript(attributes));
                this.setSubscript(StyleConstants.isSubscript(attributes));
            }
            else {
                throw new StateInvariantError("LabelView needs StyledDocument");
            }
        }
    }
    
    @Deprecated
    protected FontMetrics getFontMetrics() {
        this.sync();
        Container container = this.getContainer();
        return (container != null) ? container.getFontMetrics(this.font) : Toolkit.getDefaultToolkit().getFontMetrics(this.font);
    }
    
    @Override
    public Color getBackground() {
        this.sync();
        return this.bg;
    }
    
    @Override
    public Color getForeground() {
        this.sync();
        return this.fg;
    }
    
    @Override
    public Font getFont() {
        this.sync();
        return this.font;
    }
    
    @Override
    public boolean isUnderline() {
        this.sync();
        return this.underline;
    }
    
    @Override
    public boolean isStrikeThrough() {
        this.sync();
        return this.strike;
    }
    
    @Override
    public boolean isSubscript() {
        this.sync();
        return this.subscript;
    }
    
    @Override
    public boolean isSuperscript() {
        this.sync();
        return this.superscript;
    }
    
    @Override
    public void changedUpdate(DocumentEvent documentEvent, Shape shape, ViewFactory viewFactory) {
        this.font = null;
        super.changedUpdate(documentEvent, shape, viewFactory);
    }
}
